# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: measurements.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12measurements.proto\x12\x0especio.protoio\":\n\x0eSpectral_Shape\x12\r\n\x05start\x18\x01 \x01(\x01\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x01\x12\x0c\n\x04step\x18\x03 \x01(\x01\"~\n\nColour_SDS\x12-\n\x05shape\x18\x01 \x01(\x0b\x32\x1e.specio.protoio.Spectral_Shape\x12\x15\n\rvalues_double\x18\x02 \x03(\x01\x12\x0e\n\x06values\x18\x04 \x03(\x02\x12\x11\n\x04name\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x07\n\x05_name\",\n\tXYZ_value\x12\t\n\x01X\x18\x01 \x01(\x01\x12\t\n\x01Y\x18\x02 \x01(\x01\x12\t\n\x01Z\x18\x03 \x01(\x01\" \n\x08xy_value\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\"%\n\tcct_value\x12\x0b\n\x03\x63\x63t\x18\x01 \x01(\x01\x12\x0b\n\x03\x64uv\x18\x02 \x01(\x01\"\x1c\n\tTimestamp\x12\x0f\n\x07timestr\x18\x01 \x01(\t\"\xc8\x02\n\x0bMeasurement\x12\'\n\x03spd\x18\x01 \x01(\x0b\x32\x1a.specio.protoio.Colour_SDS\x12\x10\n\x08\x65xposure\x18\x02 \x01(\x01\x12&\n\x03XYZ\x18\x03 \x01(\x0b\x32\x19.specio.protoio.XYZ_value\x12$\n\x02xy\x18\x04 \x01(\x0b\x32\x18.specio.protoio.xy_value\x12&\n\x03\x63\x63t\x18\x05 \x01(\x0b\x32\x19.specio.protoio.cct_value\x12\x13\n\x0b\x64ominant_wl\x18\x06 \x01(\x01\x12\r\n\x05power\x18\x07 \x01(\x01\x12\x17\n\x0fspectrometer_id\x18\x08 \x01(\t\x12\'\n\x04time\x18\t \x01(\x0b\x32\x19.specio.protoio.Timestamp\x12\x15\n\x08\x61nc_data\x18\x10 \x01(\x0cH\x00\x88\x01\x01\x42\x0b\n\t_anc_data\"\x8e\x02\n\x10Measurement_List\x12\x31\n\x0cmeasurements\x18\x01 \x03(\x0b\x32\x1b.specio.protoio.Measurement\x12?\n\x0btest_colors\x18\x03 \x03(\x0b\x32*.specio.protoio.Measurement_List.TestColor\x12\r\n\x05order\x18\x02 \x03(\x05\x12\r\n\x05notes\x18\x10 \x01(\t\x12\x0e\n\x06\x61uthor\x18\x11 \x01(\t\x12\x10\n\x08location\x18\x12 \x01(\t\x12\x10\n\x08software\x18\x13 \x01(\t\x12\x11\n\tancillary\x18\x14 \x01(\x0c\x1a!\n\tTestColor\x12\t\n\x01\x63\x18\x01 \x03(\r\x12\t\n\x01\x66\x18\x02 \x03(\x02\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'measurements_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SPECTRAL_SHAPE._serialized_start=38
  _SPECTRAL_SHAPE._serialized_end=96
  _COLOUR_SDS._serialized_start=98
  _COLOUR_SDS._serialized_end=224
  _XYZ_VALUE._serialized_start=226
  _XYZ_VALUE._serialized_end=270
  _XY_VALUE._serialized_start=272
  _XY_VALUE._serialized_end=304
  _CCT_VALUE._serialized_start=306
  _CCT_VALUE._serialized_end=343
  _TIMESTAMP._serialized_start=345
  _TIMESTAMP._serialized_end=373
  _MEASUREMENT._serialized_start=376
  _MEASUREMENT._serialized_end=704
  _MEASUREMENT_LIST._serialized_start=707
  _MEASUREMENT_LIST._serialized_end=977
  _MEASUREMENT_LIST_TESTCOLOR._serialized_start=944
  _MEASUREMENT_LIST_TESTCOLOR._serialized_end=977
# @@protoc_insertion_point(module_scope)
